<?php

/**
 * @file
 * Provides helper functions used by the Mover Import module.
 */

/**
 * Gets an array of data files from the import folders where nodes/user/terms/etc
 * may be stored. For small exports the data can be saved in folders like "nodes"
 * or "users", but for large exports there should be multiple folders like
 * nodes_001, nodes_002, etc.
 */
function mover_import_scan_folders($content_type, $start = 1, $number = 1000, &$total) {
  $items = array();

  $directory = variable_get('mover_import_directory', '');
  $folders = mover_get_folders($directory, $content_type);

  $i = 0;
  foreach ($folders as $folder) {
    $files = scandir($directory . '/' . $folder);
    foreach ($files as $file) {
      if (preg_match('/([0-9]+).txt/', $file, $matches)) {
        $i++;
        if (($i >= $start) && ($i < ($start + $number))) {
          // Build an array of operations for each node
          $items[] = $folder . '/' . $file; //$matches[1];
        }
      }
    }
  }

  $total = $i;
  return $items;
}

/**
 * Manually load an old data record from the import directory. This can
 * be useful for custom import mapping scripts to be able to randomly get
 * old data as needed.
 *
 * @param string $subdirectory (
 *   The folder where the old data is saved, e.g. nodes, terms, users, etc.
 * @param int $id
 *   The unique ID for the old data, e.g. an NID, TID, UID, etc.
 *
 * @return $data object
 */
function mover_import_load_old($folder, $id) {
  $directory = variable_get('mover_import_directory', '');
  $folders = mover_get_folders($directory, $folder);
  foreach ($folders as $subfolder) {
    $filepath = $directory . '/' . $subfolder . '/' . $id . '.txt';
    if (file_exists($filepath)) {
      return mover_import_load_data_object($filepath, $folder);
    }
  }
  return FALSE;
}

/**
 * Loads an old data object (such as a node, term, user, etc.) from a file.
 *
 * @param string $filepath
 *   The full path to the data file to load.
 */
function mover_import_load_data_object($filepath) {
  // Only load files inside the import directory
  $directory = variable_get('mover_import_directory', '');
  if (strpos($filepath, $directory) === 0) {
    // Get the base folder of the sub-folder where the data is loaded from.
    $folder = explode('/', substr($filepath, strlen($directory)));
    $folder = $folder[0];
  }
  else {
    return FALSE;
  }

  // Rebuild an object from the code in the textfile being imported.
  $code = file_get_contents($filepath);
  if (empty($code)) {
    drupal_set_message(t("Empty file ignored: %filepath", array('%filepath' => $filepath)), 'error');
    return FALSE;
  }
  $data = _mover_import_decode_object($code);
  $data = _mover_import_restore_recursion($data);
  if (!is_object($data)) {
    drupal_set_message(t("Invalid object in %filepath", array('%filepath' => $filepath)), 'error');
    return FALSE;
  }

  // Give other modules a chance to massage the data, in case the
  // data being imported is in a non-standard (non-Drupal) format.
  mover_import_invoke('mover_load_data_object', $data, NULL, $folder);

  return $data;
}

/**
 * Evaluate and return decoded string.
 */
function _mover_import_decode_object($string) {

  // Allow other modules to take over this entire process.
  $return = FALSE;
  drupal_alter('mover_object_decode', $return, $string);
  if ($return !== FALSE) {
    return $return;
  }

  // Try to fix an occasional issue with double escaping of single quotes.
  // This is really an export issue, but it's nice to check the data for
  // this problem on the import too. Here's what happens with single quotes:
  // Original data:
  // $node->body = 'It's a test. It\'s a test.';
  // Exported data:
  // $node->body = 'It\'s a test. It\\'s a test.';
  // The second sentence with the double escaping will cause the PHP eval() to
  // fail.
  $string = str_replace('\\\\\'', '\\\'', $string, $count);
  if ($count) {
    drupal_set_message(t('Removed double esaping of @count single quotes in source data!',
      array('@count' => $count)), 'error');
  }

  // Default object decoding (convert arrays back to objects)
  $array = eval('return ' . $string . ';');
  if (isset($array['#_export_encode_object'])) {
    unset($array['#_export_encode_object']);
    $array = (object) $array;
  }

  return $array;
}

/**
 * Restore recursion to an object or array.
 */
function _mover_import_restore_recursion($o) {
  return unserialize(
    preg_replace(
      '#s:[0-9]+:"\x00(r|R)ecursion_export_node_([0-9]+)";#',
      '\1:\2;',
      serialize($o)
    )
  );
}

/**
 * Gets the local folder for where a CCK field stores its'
 * files. This can be different for each CCK field on each node type.
 *
 * @param $node_type
 * @param $field
 * @return string
 */
function mover_import_get_cck_file_folder($node_type, $field) {
  $info = field_info_instance('node', $field, $node_type);
  if (empty($info['settings']['file_directory'])) {
    $folder = '';
  }
  else {
    $folder = $info['settings']['file_directory'];
  }
  return $folder;
}

/**
 * Gets the new local filepath for where a file should be saved.
 * This can be different for each CCK field on each node type.
 *
 * @param $old_filepath
 *   An old filepath, in any standard form such as
 *   "sites/old.site.com/files/images/myimage.png".
 * @param $folder
 *   A folder within the files directory where the file should be saved, such as
 *   "article_images". Use mover_import_get_cck_file_folder() to get this folder
 *   name.
 *
 * @return $filepath
 *   A new filepath, such as "sites/default/files/article_images/myimage.png"
 */
function mover_import_get_new_filepath($old_filepath, $folder) {
  // Get the local directory where this file should go
  $path = 'public://';
  if ($folder) {
    $path .= $folder .'/';
  }
  file_prepare_directory($path, FILE_CREATE_DIRECTORY);

  // Parse the filepath
  $filepath = mover_unclean_local_file_path($old_filepath);
  $parts = parse_url($filepath);

  // Strip oldsite files directories from the filename
  $new_filename = $parts['path'];
  $new_filename = mover_import_trim_filepath($new_filename);
  $new_filename = trim($new_filename, '/');
  $new_filename = implode('__', explode('/', $new_filename));

  // Create the new local filepath
  $filepath = $path . $new_filename;
  return $filepath;
}

/**
 * Tries to download a file from a remote site.
 *
 * Normally all files are exported by Mover Export and then moved over to the
 * new site files directory for importing. However, as a fallbackm this function
 * will attempt to retrieve a file directly from a remote site. Note that it
 * won't download a file if it already exists locally.
 *
 * @param string $remote_url
 *   A full absolute URL of a remote file.
 * @param string $local_filepath
 *   A full local Drupal filepath where the remote file should be saved. This
 *   should be in the form of sites/default/files/image.jpg.
 *
 * @return bool
 *   Boolean value for whether or not the local file [now] exists.
 */
function mover_import_download_remote_file($remote_url, $local_filepath) {
  if (!file_exists($local_filepath)) {
    // Attempt to download the file using CURL
    $ch = curl_init();

    // Clean the URL (spaces will break the request)
    $remote_url = str_replace(' ', '%20', $remote_url);
    curl_setopt($ch, CURLOPT_URL, $remote_url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($ch, CURLOPT_BINARYTRANSFER, TRUE);
    curl_setopt($ch, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
    curl_setopt($ch, CURLOPT_AUTOREFERER, TRUE);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
    curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
    curl_setopt($ch, CURLOPT_ENCODING, "");
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);

    $output = curl_exec($ch);
    $info = curl_getinfo($ch);
    if ($info['http_code'] == '200') {
      $file = fopen($local_filepath, "w");
      fwrite($file, $output);
      fclose($file);
      drupal_set_message(t('File @filepath downloaded from the old site using CURL.', array(
        '@filepath' => basename($local_filepath),
      )));
    }
    curl_close($ch);
  }

  // If the local file exists, return it
  return (file_exists($local_filepath)) ? TRUE : FALSE;
}

/**
 * Finds out if a remote file exists by requesting the headers and returning the
 * HTTP status code.
 *
 * @param string $remote_url
 *   A full absolute URL of a remote file.
 *
 * @return int $http_status_code
 *   An HTTP status code, e.g. 404 (not found).
 */
function mover_import_get_remote_file_status($remote_url) {
  $headers = get_headers($remote_url, 1);
  preg_match('/([0-9]{3})/', $headers['0'], $match);
  $http_status_code = $match[1];
  return $http_status_code;
}

/**
 * Deletes a term and all its' sub-terms if it (and all its' sub-terms) is not
 * referenced by any nodes.
 *
 * @TODO: Update to Drupal 7
 *
 * This function is used for cleanup, and it is especially useful if an import
 * fails and a lot of extra terms are accidentally created.
 * WARNING: This isn't necessary safe to use on any site. Some sites may have
 * taxonomy data that is useful on it's only without it being used by any nodes.
 * Use caution and make a backup before any delete operations such as this one!
 *
 * @param int $tid
 *   A term ID for a term to delete (if it's not in use).
 * @param bool $include_old_revisions
 *   Whether or not to include old revisions when considering if a term has been
 *   referenced by a node.
 * @param int $count
 *   For internal use only.
 *
 * @return int $count
 *   The number of terms removed.
 */
/*function mover_import_remove_unused_term($tid, $include_old_revisions = FALSE, $count = 0) {
  if ($include_old_revisions) {
    $nid = db_query("SELECT n.nid
      FROM {taxonomy_term_node} tn
      INNER JOIN {node} n ON tn.nid=n.nid AND tn.vid=n.vid
      WHERE tn.tid = :tid", array(':tid' => $tid))->fetchField();
  }
  else {
    $nid = db_query("SELECT n.nid
      FROM {taxonomy_term_node} tn
      INNER JOIN {node} n ON tn.nid=n.nid
      WHERE tn.tid = :tid", array(':tid' => $tid))->fetchField();
  }
  if ($nid) {
    // Not an unused term!
    return FALSE;
  }
  else {
    // Make sure this isn't the parent for another term
    $result = db_query("SELECT tid FROM {taxonomy_term_hierarchy} WHERE parent = :parent", array(':parent' => $tid));
    while ($c = db_fetch_object($result)) {
      $num = mover_import_remove_unused_term($c->tid, $include_old_revisions, $count);
      if ($num) {
        $count += $num;
      }
    }

    // Try again to see if there are any child items. If not, then we can delete this term.
    $child = db_query("SELECT tid FROM {taxonomy_term_hierarchy} WHERE parent = :parent", array(':parent' => $tid))->fetchField();
    if (!$child) {
      $name = db_query("SELECT name FROM {taxonomy_term_data} WHERE tid = :tid", array(':tid' => $tid))->fetchField();
      drupal_set_message(t('Deleting empty term %term (TID @tid)', array(
        '%term' => $name,
        '@tid' => $tid,
      )));
      taxonomy_term_delete($tid);
      $count++;
    }
  }
  return $count;
}
*/