<?php

/**
 * @file
 * Handles importing of nodes.
 */

/**
 * This is a "data callback" function that is called by the import for
 * each data object loaded from an import file. This function is responsible
 * for calling the "process callback" which does the actual updating/importing
 * tasks.
 *
 * @param $data
 * @param $process
 * @param &$context
 * @return void
 */
function mover_import_nodes_process_data($data, $process, &$context) {

  // Get information about the process
  $process_info = mover_import_get_process($process);

  // Give other modules a chance to massage the data early on, in case the
  // data being imported is in a non Drupal 7 standard format.
  //mover_import_invoke('data_preprocess', $data, array(), 'nodes');

  // Start a new node object for this data
  $node = mover_node_clone($data);

  // See if this data was already imported
  if ($nid = mover_get_new_nid($data->nid)) {
    $existing_node = node_load($nid, NULL, TRUE);
    $node->nid = $existing_node->nid;
    $node->vid = $existing_node->vid;
    mover_import_invoke('node_import_update', $node, $data);
  }
  else {
    mover_import_invoke('node_import_new', $node, $data);
  }

  // Let the process callback function handle the real work
  $function = $process_info['process callback'];
  if (function_exists($function)) {
    $function($node, $data, $context);
  }
  else {
    drupal_set_message(t("Function %function not found!", array(
      '%function' => $function,
    )), 'error');
  }
}

/**
 * Process callback for importing nodes.
 *
 * "import_nodes" is the default node processing handler, defined in
 * this module by hook_mover_import_processes().
 */
function mover_import_nodes_process_import(&$node, $data, &$context) {

  // Remove control characters from the node title (they can break the whole import process and cause an unknown error!)
  $node->title = trim(preg_replace('/[\x{2028}-\x{2029}\x]+/u', '', $data->title));

  // Adjust the data structure of nodewords (at least in D6 it seems to export
  // and import using different data structures)
  $node->nodewords = array(
    'metatags' => (isset($data->nodewords)) ? $data->nodewords : array(),
  );

  // Try to load or import the same user who originally created this node
  $uid = mover_get_new_uid($data->uid);
  if (!$uid) {
    $user_data = mover_import_load_old('users', $data->uid);
    if ($user_data) {

      // See if this data was already imported
      if ($uid = mover_get_new_uid($user_data->uid, $user_data->mail, $user_data->name)) {
        // No user updates will be performed here. Run the user import/update process
        // for that.
      }

      // Start a new user object for this data
      else {
        // Start a new user account for this user
        $account = '';
        $userinfo = mover_user_clone($user_data);
      }

      // Let the process callback function handle the real work
      mover_import_users_process_import($account, $userinfo, $user_data, $user_context);
    }

    $uid = (isset($account->uid)) ? $account->uid : 0;
  }
  $account = user_load($uid);
  $node->name = $account->name;
  $node->uid = $account->uid;

  // Now let other modules process the node, or cancel the import. This is where
  // all custom field mapping should happen.
  $node->import = TRUE;
  mover_import_invoke('mover_node_import_process', $node, $data);

  // See if the import of this node was cancelled
  if ($node->import === FALSE) {
    $msg = t('Importing of node %title (%type) with old NID @old-nid was canceled by an import preprocess hook.',
      array(
      '%title' => $node->title,
      '%type' => $data->type,
      '@old-nid' => $data->nid,
    ));
    drupal_set_message($msg);
    $context['message'] = $msg;
    return;
  }

  // Find all taxonomy term references and save the data for later
  if (isset($data->taxonomy) && is_array($data->taxonomy)) {


    // Loop through all the vocabularies being imported
    $node->references['term'] = array();
    foreach ($data->taxonomy as $old_tid) {
      // The $data->taxonomy array could be a simple list of TIDs, or full term
      // data objects.
      if (!is_numeric($old_tid)) {
        $term_data = $old_tid;
        $old_tid = $term_data->tid;
      }
      $tid = mover_import_get_new_id('term', $old_tid);
      $term = taxonomy_term_load($tid);

      // Normally terms are imported before nodes, as a separate
      // import process. However, to selectively import only
      // terms that are actually used by nodes you can set this
      // $node->create_terms_on_demand flag in an import_preprocess
      // operation.
      if (!empty($node->create_terms_on_demand)) {
        if (!$term_data) {
          $term_data = mover_import_load_old_term($old_tid);
        }
        if ($term_data) {
          $term = mover_import_terms_process_data($term_data);
        }
      }

      // If we found a TID, then add it to the references array
      if (!empty($term->tid)) {
        // Figure out which taxonomy reference field this term relates to
        $fields = mover_import_get_taxonomy_term_reference_fields($node->type);
        if ($fields) {
          foreach ($fields as $field => $vocabs) {
            if (in_array($term->vocabulary_machine_name, $vocabs)) {
              $node->references['term'][$field][] = $old_tid;
            }
          }
        }

        /*
        $result = db_query("SELECT * FROM {taxonomy_vocabulary} WHERE vid = :vid", array(':vid' => $term->vid));
        $vocabulary = db_fetch_object($result);

        // On import Drupal expects the taxonomy structure to look a certain way, and
        // free-tagging vocabularies need a different format
        if ($vocabulary->tags) {
          // The term path for tags shouldn't have a hierarchy... but we're imploding anyway just
          // so we don't lose data in such a rare case that a hierarchical structure was being
          // imported to a free tagging structure.
          $new_taxonomy['tags'][$term->vid][] = $term->name;
        }
        else {
          $new_taxonomy[$term->vid][$term->tid] = $term->tid;
        }*/
      }
    }

    // Drupal expects tags to be comma separated before saving
    if (isset($new_taxonomy['tags']) && count($new_taxonomy['tags'])) {
      // Comma seperate the tags
      foreach ($new_taxonomy['tags'] as $vid => $tags) {
        $new_taxonomy['tags'][$vid] = implode(', ', $tags);
      }
    }
  }

  // Find all node and user reference fields and save the data for later
  $node->references['node'] = array();
  $node->references['user'] = array();
  foreach ($node as $field => $field_data) {
    if (is_array($field_data)) {
      foreach ($field_data as $lang => $values) {
        if (is_array($values) && is_array($values[0])) {
          $first_value = $values[0];

          // Node references
          if (!empty($first_value['nid'])) {
            $node->references['node'][$field] = array();
            foreach ($values as $value) {
              if (!empty($value['nid'])) {
                $node->references['node'][$field][] = $value['nid'];
              }
            }
            // Clear the original node array so that invalid node references
            // to old (non-existant) nodes won't be saved with the new node.
            $node->{$field} = array();
          }
          // User references (note that files also have a "uid" attribute
          if (!empty($first_value['uid']) && (!isset($first_value['filepath']))) {
            $node->references['user'][$field] = array();
            foreach ($values as $value) {
              if (!empty($value['uid'])) {
                $node->references['user'][$field][] = $value['uid'];
              }
            }
            // Clear the original user array so that invalid user references
            // to old (non-existant) users won't be saved with the new node.
            $node->{$field} = array();
          }
        }
      }
    }
  }

  // Handle node and user reference field remapping now IF ALL the data to be
  // referenced by a field already exists. Otherwise the reference mapping
  // operation will need to be run later.
  foreach ($node->references as $reference_type => $reference_data) {
    foreach ($reference_data as $field => $values) {
      $field_data = array();
      // Success is an all or nothing thing for each field. We'll assume success
      // but mark it as a failure if anything goes wrong
      $success = TRUE;
      foreach ($values as $value) {
        // This module only handles user reference and node reference fields.
        // Other modules will be need to support other types of references on nodes
        if ($reference_type == 'user') {
          if (empty($node->{$field}[LANGUAGE_NONE][0]['uid'])) {
            $node->{$field}[LANGUAGE_NONE] = array();
          }
          if ($new_id = mover_get_new_uid($value)) {
            $field_data[LANGUAGE_NONE][] = array('uid' => $new_id);
          }
          else {
            $success = FALSE;
          }
        }
        elseif ($reference_type == 'node') {
          if (empty($node->{$field}[LANGUAGE_NONE][0]['nid'])) {
            $node->{$field}[LANGUAGE_NONE] = array();
          }
          if ($new_id = mover_get_new_nid($value)) {
            $field_data[LANGUAGE_NONE][] = array('nid' => $new_id);
          }
          else {
            $success = FALSE;
          }
        }
        else {
          $success = FALSE;
        }
      }
      if ($success) {
        $node->{$field} = $field_data;
        unset($node->references[$reference_type][$field]);
      }
    }
  }

  // Save the node as long as the node type is valid
  $node_types = node_type_get_name($node->type);
  if ($node->type && $node_types) {
    node_save($node);

    // Create URL redirects for old URL aliases
    if (is_array($data->aliases)) {
      foreach ($data->aliases as $lang => $url_aliases) {
        foreach ($url_aliases as $url_alias) {
          mover_import_add_redirect($url_alias, 'node/' . $node->nid);
        }
      }
    }

    // Create URL redirects for old path redirects
    if (is_array($data->redirects)) {
      foreach ($data->redirects as $redirect) {
        mover_import_add_redirect($redirect, 'node/' . $node->nid);
      }
    }

    // Give other modules one more chance to act on this node after it has been imported
    mover_import_invoke('node_import_postprocess', $node, $data);

    $context['message'] = t('Processed old node ID @nid (%title)', array(
      '%title' => $data->title,
      '@nid' => $data->nid,
    ));
  }

  // If the node type was set, but it's not a valid node type then error
  elseif ($node->type) {
    $message = t("Unable to save %title because the node type %type doesn't exist in this installation!", array('%title' => $node->title, '%type' => $node->type));
    drupal_set_message($message, 'error');
    $context['message'] = $message;
  }

  else {
    $context['message'] = t('Skipping node %title (%type) because it was set to not be imported.', array('%title' => $node->title, '%type' => $data->type));
  }
}

/**
 * Implementation of hook_mover_node_import_process().
 */
function mover_import_mover_node_import_process(&$node, $data, $a4) {
  foreach ($node as $attr => $field_data) {
    if (is_array($field_data)) {
      foreach ($field_data as $lang => $value_data) {

        // Ignore fields that don't have files
        if (($file_data = mover_import_check_files($value_data)) == FALSE) {
          continue;
        }

        foreach ($file_data as $i => $file) {
          $result = mover_import_process_file($node, $data, $file, $attr);

          // The file was saved successfully, update the file field
          if ($result == TRUE && isset($file->fid)) {
            $node->{$attr}[$lang][$i]['fid'] = $file->fid;
            $node->{$attr}[$lang][$i]['uri'] = $file->uri;
          }
        }
      }
    }
  }
}

/**
 * Checks an array of file field data, removing any that are invalid.
 *
 * @return $files or FALSE
 *   Returns an array of $file objects, or FALSE if there are no files.
 */
function mover_import_check_files($file_field_data) {
  $files = array();

  foreach ($file_field_data as $i => $file) {
    $file = (object) $file;

    if (isset($file->uri)) {
      // NOTE: Preserving the $i key is important for later manipulation
      $files[$i] = $file;
    }
    // For migrating from Drupal 6
    elseif (isset($file->filepath)) {
      $file->uri = $file->filepath;
      unset($file->filepath);
      $files[$i] = $file;
    }
  }

  return (count($files) > 0) ? $files : FALSE;
}

/**
 * Process callback for updating links and image URLs in the body of nodes.
 */
function mover_import_update_body_links_process(&$node, $data, &$context) {
  $is_dirty = FALSE;

  // Copy any inline files that were exported with this node
  if (isset($data->inline_files) && is_array($data->inline_files)) {
    $directory = variable_get('mover_import_directory', '');
    $assets_path = $directory . '/files';
    if (mover_check_assets_path($assets_path)) {
      foreach ($data->inline_files as $file) {
        $file = (object) $file;
        if (isset($file->exported_file)) {
          $file->mover_exported_file = $file->exported_file;
        }
        mover_import_process_file($node, $data, $file, MOVER_INLINE_FILES_DIRECTORY);
      }
    }
  }

  // Search the node body for inline images and links, and fix them if possible
  $current_body = $node->body;
  $data_body = (!empty($data->body)) ? $data->body : '';
  $node->body = mover_import_process_inline_content($current_body, $data_body, $node, $data);
  if ($current_body != $node->body) {
    $is_dirty = TRUE;
  }

  // Also search the node teaser for inline images and links, IF it's different than the body
  if ($current_body != $node->teaser) {
    $current_teaser = $node->teaser;
    $data_teaser = (!empty($data->teaser)) ? $data->teaser : '';
    $node->teaser = mover_import_process_inline_content($current_teaser, $data_teaser, $node, $data);
    if ($current_teaser != $node->teaser) {
      $is_dirty = TRUE;
    }
  }
  else {
    $node->teaser = $node->body;
  }

  if ($is_dirty && !$node->disable_save && is_numeric($node->nid)) {
    node_save($node);
  }

  mover_import_increment_counter($context);
}

/**
 * Prepare a clone of the node during import.
 */
function mover_node_clone($original_node) {
  global $user;

  // Clone the original node object and start modifying it...
  $node = clone $original_node;

  // Reset the NID and VID so a new node will be saved
  $node->nid = NULL;
  $node->vid = NULL;

  // Clear the old path alias so a new one can be created
  $node->path = NULL;

  // Setup pathauto to create a new path alias.
  $node->pathauto_perform_alias = TRUE;

  // Setup a references array for temporarily storing node reference, user
  // reference, and other types of relationship data.
  $node->references = array();

  // Other things from the original node_export module that may be useful someday
  if (variable_get('mover_reset_book_mlid_' . $node->type, TRUE) && isset($node->book['mlid'])) {
    $node->book['mlid'] = NULL;
  }

  if (variable_get('mover_reset_' . $node->type, FALSE)) {
    $node_options = variable_get('node_options_' . $node->type, array('status', 'promote'));
    // Fill in the default values.
    foreach (array('status', 'moderate', 'promote', 'sticky', 'revision') as $key) {
      $node->$key = in_array($key, $node_options);
    }
  }

  // Add a couple of useful references to the $node object for the old NID and type
  $node->old_nid = $original_node->nid;
  $node->old_type = $original_node->type;

  return $node;
}

/**
 * Implements hook_node_delete().
 */
function mover_import_node_delete($node) {
  // Delete mover maps to a node if it's deleted
  $map_id = db_query("SELECT map_id FROM {mover_map} WHERE type = :type AND new_content_id = :new_content_id", array(':type' => 'node', ':new_content_id' => $node->nid))->fetchField();
  if ($map_id) {
    // TODO Please review the conversion of this statement to the D7 database API syntax.
    /* db_query("DELETE FROM {mover_map} WHERE map_id=%d", $map_id) */
    db_delete('mover_map')
  ->condition('map_id', $map_id)
  ->execute();
    // TODO Please review the conversion of this statement to the D7 database API syntax.
    /* db_query("DELETE FROM {mover_map_references} WHERE map_id=%d", $map_id) */
    db_delete('mover_map_references')
  ->condition('map_id', $map_id)
  ->execute();
  }
}

/**
 * Implements hook_node_insert().
 */
function mover_import_node_insert($node) {

  // Save a reference to this NID in the map table
  if (!empty($node->old_nid)) {
    // See if this new/old node relationship is already mapped
    $query = db_select('mover_map');
    $query->fields(NULL, array('map_id'));
    $query->condition('type', 'node');
    $query->condition('new_content_id', $node->nid);
    $query->condition('old_content_id', $node->old_nid);
    $map_id = $query->execute()->fetchField();

    if ($map_id) {
      // Change the "updated" date for existing maps
      // TODO Please review the conversion of this statement to the D7 database API syntax.
      /* db_query("UPDATE {mover_map} SET updated=%d WHERE map_id=%d", REQUEST_TIME, $map_id) */
      db_update('mover_map')
        ->fields(array(
          'updated' => REQUEST_TIME,
        ))
        ->condition('map_id', $map_id)
        ->execute();
    }
    else {
      // If this new/odd node relationship hasn't been mapped, or was mapped incorrectly, then add the map record now
      db_delete('mover_map')
        ->condition('type', 'node')
        ->condition('new_content_id', $node->nid)
        ->execute();
      db_delete('mover_map')
        ->condition('type', 'node')
        ->condition('old_content_id', $node->old_nid)
        ->execute();
      $map_id = db_insert('mover_map')
        ->fields(array(
          'type' => 'node',
          'new_content_id' => $node->nid,
          'old_content_id' => $node->old_nid,
          'imported' => REQUEST_TIME,
        ))
        ->execute();
      //$map_id = db_last_insert_id('mover_map', 'map_id');
    }
  }

  // Get the map ID for this content in case the "old NID" wasn't passed
  if (empty($map_id)) {
    $map_id = db_query("SELECT map_id FROM {mover_map}
          WHERE type = :type AND new_content_id = :new_content_id", array(':type' => 'node', ':new_content_id' => $node->nid))->fetchField();
  }

  // If we have a Map ID then save node and user references for later
  if (!empty($map_id)) {

    // Save references for processing later
    if (is_array($node->references)) {
      foreach ($node->references as $type => $references) {
        foreach ($references as $field => $ids) {
          foreach ($ids as $id) {
            mover_import_add_reference_mapping($map_id, $field, $type, $id);
          }
        }
      }
    }
  }
}

/**
 * Implements hook_node_update().
 */
function mover_import_node_update($node) {

  // Save a reference to this NID in the map table
  if (!empty($node->old_nid)) {
    // See if this new/old node relationship is already mapped
    $map_id = db_query("SELECT map_id FROM {mover_map}
          WHERE type = :type AND new_content_id = :new_content_id AND old_content_id = :old_content_id", array(
            ':type' => 'node',
            ':new_content_id' => $node->nid,
            ':old_content_id' => $node->old_nid
          ))->fetchField();
    if ($map_id) {
      // Change the "updated" date for existing maps
      db_update('mover_map')
        ->fields(array(
          'updated' => REQUEST_TIME,
        ))
        ->condition('map_id', $map_id)
        ->execute();
    }
    else {
      // If this new/odd node relationship hasn't been mapped, or was mapped incorrectly, then add the map record now
      // TODO Please review the conversion of this statement to the D7 database API syntax.
      /* db_query("DELETE FROM {mover_map} WHERE type='%s' AND (new_content_id=%d OR old_content_id=%d)", 'node', $node->nid, $node->old_nid) */
      db_delete('mover_map')
        ->condition('type', 'node')
        ->where('new_content_id=%d OR old_content_id=%d', array($node->nid, $node->old_nid))
        ->execute();
      // TODO Please review the conversion of this statement to the D7 database API syntax.
      /* db_query("INSERT INTO {mover_map} (type, new_content_id, old_content_id, imported) VALUES ('node', %d, %d, %d)", $node->nid, $node->old_nid, REQUEST_TIME) */
      $id = db_insert('mover_map')
        ->fields(array(
          'type' => 'node',
          'new_content_id' => $node->nid,
          'old_content_id' => $node->old_nid,
          'imported' => REQUEST_TIME,
        ))
        ->execute();
      $map_id = db_last_insert_id('mover_map', 'map_id');
    }
  }

  // Get the map ID for this content in case the "old NID" wasn't passed
  if (empty($map_id)) {
    $map_id = db_query("SELECT map_id FROM {mover_map}
          WHERE type = :type AND new_content_id = :new_content_id", array(':type' => 'node', ':new_content_id' => $node->nid))->fetchField();
  }

  // If we have a Map ID then save node and user references for later
  if (!empty($map_id)) {

    // Save references for processing later
    if (!empty($node->references) && is_array($node->references)) {
      foreach ($node->references as $type => $references) {
        foreach ($references as $field => $ids) {
          foreach ($ids as $id) {
            mover_import_add_reference_mapping($map_id, $field, $type, $id);
          }
        }
      }
    }
  }
}

/**
 * Implements hook_node_view().
 */
function mover_import_node_view($node, $view_mode = 'full') {
  if (($view_mode === 'full') && user_access('import content') && variable_get('mover_oldsite_base_urls', '')) {
    $old_nid = db_query("SELECT old_content_id FROM {mover_map} WHERE type = :type AND new_content_id = :new_content_id", array(':type' => 'node', ':new_content_id' => $node->nid))->fetchField();
    if ($old_nid) {
      $base_urls = mover_get_oldsite_urls();
      drupal_set_message(t('This %type was imported from <a href="@url">NID @nid</a>', array(
        '%type' => $node->type,
        '@url' => $base_urls[0] . '/node/' . $old_nid,
        '@nid' => $old_nid,
      )));
    }
  }
}

/**
 * Manually load an old node record from the import directory.
 *
 * @param $old_nid
 * @return $node object
 */
function mover_import_load_old_node($old_nid) {
  return mover_import_load_old('nodes', $old_nid);
}

/**
 * Gets the new NID for an old node record.
 *
 * @param  $old_nid
 * @param string $old_path_alias
 *   Optional alternate way to load the node mapping (via the path redirect table)
 * @return $new_nid
 */
function mover_get_new_nid($old_nid, $old_path_alias = '') {
  $nid = db_query("SELECT new_content_id FROM {mover_map} WHERE type = :type AND old_content_id = :old_content_id", array(':type' => 'node', ':old_content_id' => $old_nid))->fetchField();

  // For some legacy imports, use the $node->field_old_nid value (part of the post type)
  if (!$nid) {
    if (db_table_exists('content_type_post') && db_field_exists('content_type_post', 'field_old_nid_value')) {
      $nid = db_query("SELECT nid FROM {content_type_post} WHERE field_old_nid_value = :field_old_nid_value", array(':field_old_nid_value' => $old_nid))->fetchField();
      // Save this mapping for later
      if ($nid) {
        // TODO Please review the conversion of this statement to the D7 database API syntax.
        /* db_query("INSERT INTO {mover_map} (type, new_content_id, old_content_id) VALUES ('node', %d, %d)", $nid, $old_nid) */
        $id = db_insert('mover_map')
          ->fields(array(
            'type' => 'node',
            'new_content_id' => $nid,
            'old_content_id' => $old_nid,
          ))
          ->execute();
      }
    }
  }

  // For legacy imports, use the path redirect table to try and find the NID
  if (!$nid) {
    $redirect = db_query("SELECT redirect FROM {redirect} WHERE source = :source", array(':source' => $old_path_alias))->fetchField();
    if (preg_match('/^node\/[0-9]+$/', $redirect)) {
      $nid = str_replace('node/', '', $redirect);
      // Save this mapping for later
      // TODO Please review the conversion of this statement to the D7 database API syntax.
      /* db_query("INSERT INTO {mover_map} (type, new_content_id, old_content_id) VALUES ('node', %d, %d)", $nid, $old_nid) */
      $id = db_insert('mover_map')
        ->fields(array(
          'type' => 'node',
          'new_content_id' => $nid,
          'old_content_id' => $old_nid,
        ))
        ->execute();
    }
  }

  return $nid;
}

/**
 * Builds an array of batch operations that loops through all existing nodes
 * in the database, including nodes not imported by Mover Import.
 */
function mover_import_existing_nodes_operations($process) {
  $operations = array();

  // Variables for this run
  $start = variable_get('mover_import_start', 1);
  $number = variable_get('mover_import_number', 1000);

  // Build an array of operations for each data item
  $total = 0;
  $query = db_select('node');
  $query->fields(NULL, array('nid'));
  $query->orderBy('created', 'DESC');
  $result = $query->execute();
  foreach ($result as $n) {
    $total++;
    $operations[] = array('mover_import_existing_nodes_process_data', array($n->nid, $process));
  }

  // Save the number of items that should be / will be imported
  variable_set('mover_import_total', $total);

  return $operations;
}

/**
 * This is a "data callback" function that is called by the import for
 * each existing node object loaded. This function is responsible
 * for calling the "process callback" which does the actual updating/importing
 * tasks.
 *
 * @param $data
 * @param $process
 * @param &$context
 * @return void
 */
function mover_import_existing_nodes_process_data($nid, $process, &$context) {

  // Get information about the process
  $process_info = mover_import_get_process($process);

  // Load the node
  $node = node_load($nid, NULL, TRUE);

  // Load the old node data, if possible
  $old_nid = db_query("SELECT old_content_id
    FROM {mover_map}
    WHERE type = :type AND new_content_id = :new_content_id", array(
      ':type' => 'node',
      ':new_content_id' => $nid
    ))->fetchField();
  $data = mover_import_load_old_node($old_nid);
  if (!$data) {
    $data = new stdClass();
  }

  // Give other modules a chance to massage the data early on, in case the
  // data being imported is in a non-standard (non-Drupal) format.
  //mover_import_invoke('data_preprocess', $data, array(), 'nodes');

  // Let the process callback function handle the real work
  $function = $process_info['process callback'];
  if (function_exists($function)) {
    $function($node, $data, $context);
  }
  else {
    drupal_set_message(t("Function %function not found!", array(
      '%function' => $function,
    )), 'error');
  }
}
