<?php

/**
 * @file
 * Defines page handlers and forms provided by the Mover Import module.
 */

/**
 * Main form callback for importing content.
 */
function mover_import_form($form, &$form_state) {
  $form['directory'] = array(
    '#type' => 'textfield',
    '#title' => t('Import file directory'),
    '#default_value' => variable_get('mover_import_directory', file_directory_path() . '/import'),
    '#description' => t("This folder should contain a 'nodes' folder for the
      individual the node objects to import and a 'files' folder
      which should be a complete copy of the old site's files folder."),
    '#required' => TRUE,
  );

  // Invoke hook_mover_import_processes to get a list of processes options
  $form['process'] = array();
  $form['process']['import_process'] = array(
    '#type' => 'fieldset',
    '#title' => t('Import processes'),
    '#description' => t('Run these processes (in any order) to import and update content'),
  );
  $form['process']['post_process'] = array(
    '#type' => 'fieldset',
    '#title' => t('Post-import processes'),
    '#description' => t('Run any or all of these post-processing operations after all content is imported.'),
  );
  $processes = module_invoke_all('mover_import_processes');
  foreach ($processes as $process => $info) {
    $form['process'][$info['group']]['process_'. $process] = array(
      '#type' => 'radio',
      '#title' => $info['title'],
      '#return_value' => $process,
      '#default_value' => ($process == variable_get('mover_import_process', 'import_users')),
      '#parents' => array('process'),
    );
  }

  $form['start'] = array(
    '#type' => 'textfield',
    '#field_prefix' => t('Start at item number: '),
    '#default_value' => variable_get('mover_import_start', 1),
    '#size' => 8,
  );

  $form['number'] = array(
    '#type' => 'textfield',
    '#field_prefix' => t('Number of items to process:'),
    '#default_value' => variable_get('mover_import_number', 1000),
    '#size' => 8,
  );

  $form['checklist'] = array(
    '#value' => '<ol><h4>Before importing, have you:</h4>
      <li>Copied the exported data folders to the import directory below?</li>
      <li>Enabled any modules or features that relate to content fields?</li>
      <li>Manually setup user profile fields and roles that need to be migrated?</li>
      </ol>',
  );

  $form['import'] = array(
    '#type' => 'submit',
    '#value' => t('Import'),
  );

  return $form;
}

/**
 * Validation handler for the import form.
 */
function mover_import_form_validate($form, &$form_state) {
  // Check the import directory
  $directory = rtrim($form_state['values']['directory'], '/\\');
  if (!is_dir($directory)) {
    form_set_error('directory', t('The directory %directory does not exist.', array('%directory' => $directory)));
  }
}

/**
 * Submit handler for the import form.
 */
function mover_import_form_submit($form, &$form_state) {
  $directory = $form_state['values']['directory'];
  $process = $form_state['values']['process'];
  $start = $form_state['values']['start'];
  $number = $form_state['values']['number'];

  // Save settings for next time
  variable_set('mover_import_directory', $directory);
  variable_set('mover_import_process', $process);
  variable_set('mover_import_start', $start);
  variable_set('mover_import_number', $number);

  // Get information about the processes
  $process_info = mover_import_get_process($process);

  // Certain types of processes can be handled internally (by this module)
  // Other types of processes are handled externally by other modules
  $function = $process_info['operations callback'];
  if (function_exists($function)) {
    $operations = $function($process);
  }
  else {
    drupal_set_message(t("Operations function %function doesn't exist!", array(
      '%function' => $function,
    )), 'error');
  }

  // Run operations now
  if ($operations) {
    $batch = array(
      'title' => t('Content Processing'),
      'operations' => $operations,
      'finished' => 'mover_import_finished',
    );
    batch_set($batch);
  }
  else {
    drupal_set_message(t("There isn't any content to process for this operation!"));
  }
}

/**
 * @TODO: Convert this to Drupal 7
 *
 * Form callback for removing all unused terms within a vocabulary.
 */
/*function mover_import_remove_unused_terms_form($form, &$form_state) {
  $options = array();
  $result = db_query("SELECT vid, name FROM {taxonomy_vocabulary} ORDER BY name");
  while ($v = db_fetch_object($result)) {
    $options[$v->vid] = $v->name;
  }

  $form['vocabs'] = array(
    '#title' => t('Delete unused taxonomy terms from the following vocabularies'),
    '#type' => 'checkboxes',
    '#options' => $options,
  );
  $form['include_old_revisions'] = array(
    '#title' => t('Also delete taxonomy terms only used by old revisions of nodes?'),
    '#type' => 'checkbox',
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
  );
  return $form;
}
*/

/**
 * @TODO: Convert this to Drupal 7
 *
 * Submit handler for the form for removing all unused terms within a vocabulary.
 */
/*function mover_import_remove_unused_terms_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $vids = $values['vocabs'];
  $include_old_revisions = $values['include_old_revisions'];
  $count = 0;
  foreach ($vids as $vid) {
    $result = db_query("SELECT tid, name FROM {taxonomy_term_data} WHERE vid = :vid ORDER BY tid", array(':vid' => $vid));
    while ($t = db_fetch_object($result)) {
      $num = mover_import_remove_unused_term($t->tid, $include_old_revisions);
      if ($num) {
        $count += $num;
      }
    }
  }

  drupal_set_message(t('Removed @num unused terms!', array('@num' => $count)));
}
*/

/**
 * @TODO: Convert to Drupal 7.
 *
 * Form callback for merging all duplicate terms within a vocabulary.
 */
/*
function mover_import_merge_duplicate_terms_form($form, &$form_state) {
  $options = array();
  $result = db_query("SELECT vid, name FROM {taxonomy_vocabulary} ORDER BY name");
  while ($v = db_fetch_object($result)) {
    $options[$v->vid] = $v->name;
  }

  $form['vocabs'] = array(
    '#title' => t('Merge duplicate taxonomy terms from the following vocabularies. (Duplicate terms are defined as terms with the same name at the same position in the term hierarchy. This process should only be run if something went wrong with a content import. You should also make a backup of your site first!)'),
    '#type' => 'checkboxes',
    '#options' => $options,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Merge'),
  );
  return $form;
}
*/

/**
 * @TODO: Convert to Drupal 7.
 *
 * Submit handler for merging all duplicate terms within a vocabulary.
 */
/*function mover_import_merge_duplicate_terms_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $vids = $values['vocabs'];
  $count = 0;
  foreach ($vids as $vid) {
    // Find each term name that has duplicates
    $result = db_query("SELECT td.name, th.parent, COUNT(td.tid)
      FROM {taxonomy_term_data} td
      INNER JOIN {taxonomy_term_hierarchy} th ON th.tid=td.tid
      WHERE vid = :vid
      GROUP BY td.name, th.parent
      HAVING COUNT(td.tid) > :COUNT(td.tid)
      ORDER BY td.name", array(':vid' => $vid, ':COUNT(td.tid)' => 1));
    while ($r = db_fetch_object($result)) {
      // Get all the TIDs of these duplicates
      $result2 = db_query("SELECT td.tid
        FROM {taxonomy_term_data} td
        INNER JOIN {taxonomy_term_hierarchy} th ON th.tid=td.tid
        WHERE td.name = :td.name AND th.parent = :th.parent AND td.vid = :td.vid
        ORDER BY td.tid", array(':td.name' => $r->name, ':th.parent' => $r->parent, ':td.vid' => $vid));
      $target_tid = 0;
      while ($t = db_fetch_object($result2)) {
        // Save the first TID as the target that others will be merged into
        if (!$target_tid) {
          $target_tid = $t->tid;
        }

        // Remap nodes to the target TID, and delete the duplicate terms
        else {
          db_update('taxonomy_term_node')
            ->fields(array(
            'tid' => $target_tid,
            ))
            ->condition('tid', $t->tid)
            ->execute();
          taxonomy_term_delete($t->tid);
          $count++;
        }
      }
    }
  }

  drupal_set_message(t('Merged terms and removed @num duplicates!', array('@num' => $count)));
}
*/

/**
 * Lists new content that we have data files for, but has not been imported.
 */
function mover_import_list_new_content() {
  $not_imported = array();
  $total = 0;
  $data_files = mover_import_scan_folders('nodes', 1, 99999, $total);
  foreach ($data_files as $data_file) {
    $old_nid = str_replace('.txt', '', basename($data_file));
    $new_nid = mover_import_get_new_id('node', $old_nid);
    if (!$new_nid) {
      $old_node = mover_import_load_old_node($old_nid);
      $not_imported[] = "<li>Old NID $old_nid: $old_node->title</a></li>";
    }
  }
  $output = '<p><strong>Nodes not yet imported: </strong><ul>' . implode("\n", $not_imported) . '</ul></p>';
  $output .= '<p><strong>Total: </strong>' . count($not_imported) . '</p>';
  return $output;
}

/**
 * Prints the priginal (imported) data structure of the current node/user.
 */
function mover_import_load_object($object, $name = NULL) {
  $title = isset($object->title) ? $object->title : $object->name;
  drupal_set_title($title);

  // Get the original node object
  $new_nid = $object->nid;
  $old_nid = db_query("SELECT old_content_id FROM {mover_map} WHERE type = :type AND new_content_id = :new_content_id", array(':type' => 'node', ':new_content_id' => $object->nid))->fetchField();
  if ($old_nid) {
    $old_node = mover_import_load_old_node($old_nid);
    if ($old_node) {
      return kdevel_print_object($old_node, '$' . $name . '->');
    }
    else {
      return '<p>' . t('This node was imported from old NID @nid but the original data file can no longer be found.', array(
        '@nid' => $old_nid,
      )) . '</p>';
    }
  }
  else {
    return '<p>' . t('This node was not imported with the Mover module.') . '</p>';
  }
}
