<?php

/**
 * @file
 * Contains functions related to the import process itself, i.e. the process of
 * looping through data and performing operations.
 */


/**
 * Implements hook_mover_import_processes().
 *
 * Defines custom processes that can be run on imported data. This hook can be
 * used in other modules to add additional custom import processors. Each item
 * has a "type" value which gives the processors some default behaviors, so
 * it's easier to add additional processors to do specific things.
 *
 * For a normal migration that goes well you won't need this function. But if
 * something goes wrong, or you have a very unique migration, you might want
 * to do additional processing on imported data in an entirely separate process.
 * This is especially useful for quickly fixing specific things in an import
 * without updating everything, or re-importing everything. To do this you
 * should copy something like "import_nodes". Rename it and change the
 * 'process callback' to a new custom function. Then that function will
 * get called for each $node and you can do whatever you want with the $node
 * and original $data object.
 */
function mover_import_mover_import_processes() {
  return array(
    'import_users' => array(
      'title' => t('Import and update users'),
      'folder' => 'users',
      'operations callback' => 'mover_import_build_operations',
      'data callback' => 'mover_import_users_process_data',
      'process callback' => 'mover_import_users_process_import',
      'group' => 'import_process',
    ),
    'import_terms' => array(
      'title' => t('Import and update terms'),
      'folder' => 'terms',
      'operations callback' => 'mover_import_build_operations',
      'data callback' => 'mover_import_terms_process_data',
      'process callback' => 'mover_import_terms_process_import',
      'group' => 'import_process',
    ),
    'import_nodes' => array(
      'title' => t('Import and update nodes'),
      'folder' => 'nodes',
      'operations callback' => 'mover_import_build_operations',
      'data callback' => 'mover_import_nodes_process_data',
      'process callback' => 'mover_import_nodes_process_import',
      'group' => 'import_process',
    ),
    'import_aliases' => array(
      'title' => t('Import and update aliases'),
      'folder' => 'aliases',
      'operations callback' => 'mover_import_build_operations',
      'data callback' => 'mover_import_data_object_process',
      'process callback' => 'mover_import_aliases_process_import',
      'group' => 'post_process',
    ),
    'import_redirects' => array(
      'title' => t('Import and update redirects'),
      'folder' => 'redirects',
      'operations callback' => 'mover_import_build_operations',
      'data callback' => 'mover_import_data_object_process',
      'process callback' => 'mover_import_redirects_process_import',
      'group' => 'post_process',
    ),
    'map_references' => array(
      'title' => t('Re-map references (must be run after all importing)'),
      'operations callback' => 'mover_import_references_operations',
      'process callback' => 'mover_import_references_process',
      'group' => 'post_process',
    ),
    'update_body_links' => array(
      'title' => t('Update links and image URLs in the body of nodes'),
      'operations callback' => 'mover_import_existing_nodes_operations',
      'data callback' => 'mover_import_existing_nodes_process_data',
      'process callback' => 'mover_import_update_body_links_process',
      'group' => 'post_process',
    ),
  );
}

/**
 * Gets information about a specific process configuration.
 *
 * @param $process
 * @return array
 */
function mover_import_get_process($process) {
  // Get information about the various processes that can be run
  $processes = module_invoke_all('mover_import_processes');

  // Get the current process
  $process_info = $processes[$process];

  return $process_info;
}

/**
 * Builds an array of batch operations that loops through all data
 * in the specified import directory and let's other functions and other modules process them.
 */
function mover_import_build_operations($process) {
  $operations = array();

  // Variables for this run
  $start = variable_get('mover_import_start', 1);
  $number = variable_get('mover_import_number', 1000);

  // Get information about the process
  $process_info = mover_import_get_process($process);

  // Build an array of operations for each data item
  $data_files = mover_import_scan_folders($process_info['folder'], $start, $number, $total);
  foreach ($data_files as $file) {
    $operations[] = array('mover_import_data_file_process', array($file, $process));
  }

  // Save the number of items that should be / will be imported
  variable_set('mover_import_total', $total);

  return $operations;
}

/**
 * Batch processing handler for importing data from an export file.
 */
function mover_import_data_file_process($file, $process, &$context) {

  // Get information about the process
  $process_info = mover_import_get_process($process);

  $directory = variable_get('mover_import_directory', '');
  $filepath = $directory . '/' . $file;
  $data = mover_import_load_data_object($filepath, $process_info['folder']);
  if ($data) {
    $function = $process_info['data callback'];
    if (function_exists($function)) {
      $function($data, $process, $context);
    }
    else {
      drupal_set_message(t('Function %function not found!', array(
        '%function' => $function,
      )), 'error');
    }
  }
  mover_import_increment_counter($context);
}

/**
 * This is a generic "data callback" function that is called by the import for
 * each data object loaded from an import file. This function is responsible
 * for calling the "process callback" which does the actual updating/importing
 * tasks.
 *
 * @param $data
 * @param $process
 * @param &$context
 * @return void
 */
function mover_import_data_object_process($data, $process, &$context) {

  // Get information about the process
  $process_info = mover_import_get_process($process);

  // Let the process callback function handle the real work
  $function = $process_info['process callback'];
  if (function_exists($function)) {
    $function($data, $context);
  }
  else {
    drupal_set_message(t("Function %function not found!", array(
      '%function' => $function,
    )), 'error');
  }
}

/**
 * After each item in a batch process is processed, even if unsuccessful,
 * this function should be called so that the "start" variable is set
 * correctly and the processed can be resumed if the batch process fails.
 */
function mover_import_increment_counter(&$context, $name = '') {
  if ($context) {
    // Update the "start" variable for starting the next import
    $start = variable_get('mover_import_start', 1);
    $total = variable_get('mover_import_total', 0);
    $start = ($total <= $start) ? 1 : $start + 1;
    variable_set('mover_import_start', $start);
    $context['results'][] = $name;
  }
}

/**
 * This is the final batch import operation.
 *
 * This should always be run after every process to display status messages
 * about the result of the process.
 *
 * @param bool $success
 *   Whether or not the operation completed normally or failed.
 * @param array $results
 *   An array of all the items that were processed during the operation.
 * @param $operations
 *   @TODO: What does this do?
 */
function mover_import_finished($success, $results, $operations) {
  if ($success) {
    $message = format_plural(count($results), 'One item processed.', '@count items processed.');
  }
  else {
    $message = t('Finished with an error.');
  }
  drupal_set_message($message);

  // Any more items to import?
  if ((variable_get('mover_import_start', 1) > 1) &&
    (variable_get('mover_import_start', 1) < variable_get('mover_import_total', 0))) {
    drupal_set_message(t("There are still @num items left to process. Run this process again to continue.",
      array('@num' => variable_get('mover_import_total', 0) - variable_get('mover_import_start', 1) + 1)));
  }
}
