<?php

/**
 * @file
 * Handles importing of referenced data (e.g. data from node reference and user
 * reference CCK fields).
 */

/**
 * Builds an array of batch operations that loops through all unprocessed
 * reference maps and let's other functions and other modules process them.
 */
function mover_import_references_operations($process) {
  $operations = array();

  // Variables for this run
  $start = variable_get('mover_import_start', 1);
  $number = variable_get('mover_import_number', 1000);
  $i = 0;

  // Get information about the process
  $process_info = mover_import_get_process($process);

  // Re-Map references
  // This operation links up node references, user references, and maybe other types of custom references!
  // This couldn't be done earlier because the nodes/user/etc that are being references may not have been
  // imported before now.
  //
  // Start by getting the types of content that need references mapping (node, user, etc)
  /*$result = db_query("SELECT DISTINCT m.type FROM {mover_map_references} mr
      INNER JOIN {mover_map} m ON m.map_id=mr.map_id
      WHERE mr.complete = :mr.complete", array(':mr.complete' => 0));*/
  $query = db_select('mover_map_references', 'mr');
  $query->join('mover_map', 'm', 'm.map_id = mr.map_id');
  $query->fields('m', array('type'));
  $query->conditions('mr.complete', 0);
  $result = $query->execute();
  foreach ($result as $types) {
    $type = $types->type; // node, user, term, etc.

    // Now get all the nodes, users, etc that have references which need mapping. We're
    // building a list of operations for NIDs, UIDs, etc that will need to be updated.
    /*$result = db_select("SELECT DISTINCT m.new_content_id FROM {mover_map_references} mr
      INNER JOIN {mover_map} m ON m.map_id=mr.map_id
      WHERE mr.complete = :mr.complete AND m.type = :m.type
      ORDER BY m.new_content_id", array(':mr.complete' => 0, ':m.type' => $type));*/
    $query = db_select('mover_map_references', 'mr');
    $query->join('mover_map', 'm', 'm.map_id = mr.map_id');
    $query->fields('m', array('new_content_id'));
    $query->condition('mr.complete', 0);
    $query->condition('m.type', $type);
    $query->orderBy('m.new_content_id');
    $result2 = $query->execute();
    foreach ($result2 as $content_ids) {
      $id = $content_ids->new_content_id;

      // Now build a batch operations queue
      $i++;
      if ($i <= $number) {
        $function = $process_info['process callback'];
        if (function_exists($function)) {
          $operations[] = array($function, array($type, $id));
        }
        else {
          drupal_set_message(t('Function %function not found!', array(
            '%function' => $function,
          )), 'error');
        }
      }
    }
  }

  // Save the number of items that should be / will be imported
  variable_set('mover_import_total', $i);

  return $operations;
}

/**
 * References processor for nodes (not just node references, but also user references and any other
 * type of reference that a node might have to another piece of content). This function is
 * called after all other importing is done, so that the referenced content IDs will be valid.
 *
 * @param $type
 *   The type of content which has references to be updated, e.g. node, or user
 * @param $id
 *   The id of the content which has references to be updated, e.g. a NID or UID
 */
function mover_import_references_process($type, $id, &$context) {

  // Load references to be mapped for this content
  $references = array();
  /*$result = db_query("SELECT mr.* FROM {mover_map_references} mr
    INNER JOIN {mover_map} m ON m.map_id=mr.map_id
    WHERE m.type = :m.type AND m.new_content_id = :m.new_content_id
    ORDER BY reference_type, reference_id", array(':m.type' => $type, ':m.new_content_id' => $id));*/
  $query = db_select('mover_map_references', 'mr');
  $query->join('mover_map', 'm', 'm.map_id=mr.map_id');
  $query->fields('mr');
  $query->condition('m.type', $type);
  $query->condition('m.new_content_id', $id);
  $query->orderBy('reference_type');
  $query->orderBy('reference_id');
  $result = $query->execute();
  foreach ($result as $data) {
    $map_id = $data->map_id;
    $references[$data->reference_type][$data->field][] = $data->old_reference_id;
  }

  // This shouldn't happen, because of how this function is called, but just to
  // be safe...
  if (!$map_id) {
    drupal_set_message(t('No map ID for reference mapping of @type @id', array(
      '@type' => $type,
      '@id' => $id,
    )), 'error');
    mover_import_increment_counter($context);
    return;
  }

  // Let other modules (including this module) process reference mapping
  module_invoke_all('mover_import_reference_mapping', $map_id, $type, $id, $references);

  //drupal_set_message(t('Updated references for @type @id', array('@type' => $type, '@id' => $id)));
  $context['message'] = t('Updated references for @type @id', array('@type' => $type, '@id' => $id));
  mover_import_increment_counter($context);
}

/**
 * Implements hook_mover_import_reference_mapping().
 *
 * This module handles node reference and user reference mapping. Other
 * modules can implement this hook to handle other types of references.
 */
function mover_import_mover_import_reference_mapping($map_id, $type, $id, $references) {
  switch ($type) {
    case 'node':
      // Process node and user reference fields on nodes
      if (is_array($references['node']) || is_array($references['user'])) {
        $node = node_load($id, NULL, TRUE);
        mover_import_update_node_references($node, $references);
        node_save($node);

        // Mark all these references as complete
        db_update('mover_map_references')
          ->fields(array(
            'complete' => 1,
          ))
          ->condition('map_id', $map_id)
          ->execute();
      }
      break;
    case 'user':
      // Process reference fields on users
      // @TODO: Write the code for this
      if (is_array($references['user'])) {

      }
      break;
  }
}

function mover_import_update_node_references(&$node, $references) {
  foreach ($references as $reference_type => $fields) {
    foreach ($fields as $field => $values) {
      foreach ($values as $value) {
        // This module only handles user reference, node reference, and taxonomy
        // reference fields. Other modules will be need to support other types
        // of references on nodes.
        // @TODO: Extend reference mapping to support other languages
        $key = '';
        switch ($reference_type) {
          case 'user':
            $key = 'uid';
            break;
          case 'node':
            $key = 'nid';
            break;
          case 'term':
            $key = 'tid';
            break;
        }
        if ($key) {
          if (empty($node->{$field}[LANGUAGE_NONE][0][$key])) {
            $node->{$field}[LANGUAGE_NONE] = array();
          }
          $content_id = mover_import_get_new_id($reference_type, $value);
          if ($content_id) {
            $node->{$field}[LANGUAGE_NONE][] = array($key => $content_id);
          }
        }
      }
    }
  }
}
