<?php

/**
 * @file
 * Handles importing of users.
 */

/**
 * This is a "data callback" function that is called by the import for
 * each data object loaded from an import file. This function is responsible
 * for calling the "process callback" which does the actual updating/importing
 * tasks.
 *
 * @param $data
 * @param $process
 * @param &$context
 * @return void
 */
function mover_import_users_process_data($data, $process, &$context) {

  // Get information about the process
  $process_info = mover_import_get_process($process);

  // Give other modules a chance to massage the data earlier on, in case the
  // data being imported is in a non-standard (non-Drupal) format.
  //mover_import_invoke('data_preprocess', $data, array(), 'users');

  // See if this data was already imported
  if ($uid = mover_get_new_uid($data->uid, $data->mail, $data->name)) {
    $account = user_load($uid);
    $userinfo = array();
  }

  // Start a new user object for this data
  else {
    // Start a new user account for this user
    $account = '';
    $userinfo = mover_user_clone($data);
  }

  // Let the process callback function handle the real work
  $function = $process_info['process callback'];
  if (function_exists($function)) {
    $function($account, $userinfo, $data, $context);
  }
  else {
    drupal_set_message(t("Function %function not found!", array(
      '%function' => $function,
    )), 'error');
  }
}

/**
 * User import process handles the import of one user from the specified import file.
 */
function mover_import_users_process_import(&$account, $userinfo, $data, &$context) {

  // Don't import or update UID 0 or UID 1
  if (($account && $account->uid == 0) || ($data->uid == 0)) {
    drupal_set_message(t('Skipping user 0 (anonymous)'));
    return;
  }
  if (($account && $account->uid == 1) || ($data->uid == 1)) {
    drupal_set_message(t('Skipping user 1 (admin)'));
    return;
  }

  // Remap account roles
  $new_roles = array();
  if (is_array($data->roles)) {
    foreach ($data->roles as $rid => $role_name) {
      // Translate role names for some sites
      $role_names = mover_import_invoke('import_role', $userinfo, $data, $role_name);
      if (isset($role_names[0])) {
        $role_name = $role_names[0];
      }

      // Lookup the role name in the DB to get it's new role ID
      $query = db_select('role');
      $query->fields(NULL, array('rid'));
      $query->condition('name', $role_name);
      $rid = $query->execute()->fetchField();
      if ($rid) {
        $new_roles[$rid] = $role_name;
      }
    }
  }
  $userinfo['roles'] = $new_roles;

  // @TODO: Import the actual "profile_fields" definition
  // (this will need to be somehow included in the export)

  // Disable role change notification
  $role_notification_settings = array();
  $query = db_select('role');
  $query->addField(NULL, 'rid');
  $query->orderBy('rid');
  $result = $query->execute();
  foreach ($result as $r) {
    $roleid = $r->rid;
    $role_notification_settings[$roleid] = variable_get("role_change_notify_$roleid", FALSE);
    if ($role_notification_settings[$roleid]) {
      variable_set("role_change_notify_$roleid", FALSE);
    }
  }

  // Once more chance for other modules to alter the imported object
  mover_import_invoke('user_import_process', $userinfo, $account, $data);

  // Make sure we are importing or updating this user
  if ($userinfo['import'] === FALSE) {
    $context['message'] = t('User import/update cancelled for user %name', array('%name' => $data->name));
    return;
  }

  // Save the account
  $account = user_save($account, $userinfo);

  // Profile fields that aren't set to show on the registration page won't be
  // saved with a user save of a new user, so manually run the profile save again:
  if (function_exists('profile_categories')) {
    $profile_categories = profile_categories();
    foreach ($profile_categories as $profile_category) {
      profile_save_profile($userinfo, $account, $profile_category['name']);
    }
  }

  if ($account->uid) {
    // Set the password for this user to what it was before
    if (!empty($data->pass)) {
      // TODO Please review the conversion of this statement to the D7 database API syntax.
      /* db_query("UPDATE {users} SET pass='%s' WHERE uid=%d", $data->pass, $account->uid) */
      db_update('users')
  ->fields(array(
    'pass' => $data->pass,
  ))
  ->condition('uid', $account->uid)
  ->execute();
    }

    // Save this mapping for later
    $map_id = db_query("SELECT map_id FROM {mover_map} WHERE type = :type AND new_content_id = :new_content_id", array(':type' => 'user', ':new_content_id' => $account->uid))->fetchField();
    if (!$map_id) {
      // TODO Please review the conversion of this statement to the D7 database API syntax.
      /* db_query("INSERT INTO {mover_map} (type, new_content_id, old_content_id, imported) VALUES ('user', %d, %d, %d)", $account->uid, $data->uid, REQUEST_TIME) */
      $id = db_insert('mover_map')
  ->fields(array(
    'type' => 'user',
    'new_content_id' => $account->uid,
    'old_content_id' => $data->uid,
    'imported' => REQUEST_TIME,
  ))
  ->execute();
    }

    $context['message'] = 'Processed user: ' . $account->name;
  }

  // Re-enable role change notification
  foreach ($role_notification_settings as $roleid => $value) {
    if ($value) {
      variable_set("role_change_notify_$roleid", TRUE);
    }
  }
}

/**
 * Prepare a clone of a user during import.
 */
function mover_user_clone($original_user) {
  $account = clone $original_user;
  unset($account->uid);
  unset($account->data);
  unset($account->roles);

  $account = (array) $account;
  return $account;
}

/**
 * Gets the new UID for an old user record.
 *
 * @param  $old_uid
 * @param string $old_email
 *   Optional alternate way to load the user mapping
 * @param string $old_name
 *   Optional alternate way to load the user mapping
 * @return $new_nid
 */
function mover_get_new_uid($old_uid, $old_email = '', $old_name = '') {
  $uid = db_query("SELECT new_content_id FROM {mover_map} WHERE type = :type AND old_content_id = :old_content_id", array(':type' => 'user', ':old_content_id' => $old_uid))->fetchField();

  // Try to load the user based on user name (usernames are always unique)
  if (!$uid && $old_name) {
    $uid = db_query("SELECT uid FROM {users} WHERE name = :name", array(':name' => $old_name))->fetchField();
    // Save this mapping for later
    if ($uid) {
      // TODO Please review the conversion of this statement to the D7 database API syntax.
      /* db_query("INSERT INTO {mover_map} (type, new_content_id, old_content_id) VALUES ('user', %d, %d)", $uid, $old_uid) */
      $id = db_insert('mover_map')
        ->fields(array(
          'type' => 'user',
          'new_content_id' => $uid,
          'old_content_id' => $old_uid,
        ))
        ->execute();
    }
  }

  // Try to load the user based on email address (there can be duplicates)
  if (!$uid && $old_email) {
    $uid = db_query("SELECT uid FROM {users} WHERE mail = :mail ORDER BY access DESC", array(':mail' => $old_email))->fetchField();
    // Save this mapping for later
    if ($uid) {
      // TODO Please review the conversion of this statement to the D7 database API syntax.
      /* db_query("INSERT INTO {mover_map} (type, new_content_id, old_content_id) VALUES ('user', %d, %d)", $uid, $old_uid) */
      $id = db_insert('mover_map')
  ->fields(array(
    'type' => 'user',
    'new_content_id' => $uid,
    'old_content_id' => $old_uid,
  ))
  ->execute();
    }
  }

  return $uid;
}
